/* MOUSE.C
 * Interfaccia con Mouse Driver
 */

#include	<dos.h>
#include	"mouse.h"

static char NoMouse;

/* Only checks if Mouse is present */
int MouseOpen (void)
{
int far *md;

	md = (int *)(0x33L * 4);
	NoMouse = (md [0] == 0 || md [1] == 0);
	return (NoMouse);
}

/* Resets the driver;
 * If the driver or the mouse can't be initialized returns 0,
 * otherwise it returns the number of buttons in the mouse */
int MoReset (void)
{
union REGS inregs;
	if (NoMouse) return (0);
	inregs.x.ax = 0;
	int86 (0x33,&inregs,&inregs);
	if (inregs.x.ax) return (inregs.x.bx);
	else return (0);
}

/* Increments the internal mouse flag and if it is 0 then
 * shows the cursor */
void MoShowCursor (void)
{
union REGS inregs;
	if (NoMouse) return;
	inregs.x.ax = 1;
	int86 (0x33,&inregs,&inregs);
}

/* Decrements the internal mouse flag and removes the cursor.
 * Remember that multiple calls to thi function must be balanced
 * by an equal number of calls to MoShowCursor.
 * Always hide the cursor prior to draw anything on the screen */
void MoHideCursor (void)
{
union REGS inregs;
	if (NoMouse) return;
	inregs.x.ax = 2;
	int86 (0x33,&inregs,&inregs);
}

/* Puts in Pos structure current mouse position and button status
 * Count is always set to 0 */
void MoGetMousePos (MPos *Pos)
{
union REGS inregs;
	if (NoMouse)
	{
		Pos -> X = 0;
		Pos -> Y = 0;
		Pos -> Buttons = 0;
		Pos -> Count = 0;
		return;
	}
	inregs.x.ax = 3;
	int86 (0x33,&inregs,&inregs);
	Pos -> X = inregs.x.cx;
	Pos -> Y = inregs.x.dx;
	Pos -> Buttons = inregs.x.bx;
	Pos -> Count = 0;
}

/* Sets Mouse Current Cursor Position */
void MoSetMousePos (int x,int y)
{
union REGS inregs;
	if (NoMouse) return;
	inregs.x.ax = 4;
	inregs.x.cx = x;
	inregs.x.dx = y;
	int86 (0x33,&inregs,&inregs);
}

/* Puts in Pos structure current mouse position and button status
 * Count is always set to the number of presses for the selected button
 * since the last call to this function
 * The returned position is relative to the last press */
void MoGetPressInfo (MPos *Pos,int button)
{
union REGS inregs;
	if (NoMouse)
	{
		Pos -> X = 0;
		Pos -> Y = 0;
		Pos -> Buttons = 0;
		Pos -> Count = 0;
		return;
	}
	inregs.x.ax = 5;
	inregs.x.bx = button;
	int86 (0x33,&inregs,&inregs);
	Pos -> X = inregs.x.cx;
	Pos -> Y = inregs.x.dx;
	Pos -> Buttons = inregs.x.ax;
	Pos -> Count = inregs.x.bx;
}

/* Puts in Pos structure current mouse position and button status
 * Count is set to the number of releases for the selected button
 * since the last call to this function
 * The returned position is relative to the last release */
void MoGetRelInfo (MPos *Pos,int button)
{
union REGS inregs;
	if (NoMouse)
	{
		Pos -> X = 0;
		Pos -> Y = 0;
		Pos -> Buttons = 0;
		Pos -> Count = 0;
		return;
	}
	inregs.x.bx = button;
	inregs.x.ax = 6;
	int86 (0x33,&inregs,&inregs);
	Pos -> X = inregs.x.cx;
	Pos -> Y = inregs.x.dx;
	Pos -> Buttons = inregs.x.ax;
	Pos -> Count = inregs.x.bx;
}

/* Sets the minimum and maximum horizontl position.
 * The mouse cursor can't be moved outside that range */
void MoSetHorRange (int min,int max)
{
union REGS inregs;
	if (NoMouse) return;
	inregs.x.ax = 7;
	inregs.x.cx = min;
	inregs.x.dx = max;
	int86 (0x33,&inregs,&inregs);
}

/* Sets the minimum and maximum vertical position.
 * The mouse cursor can't be moved outside that range */
void MoSetVerRange (int min,int max)
{
union REGS inregs;
	if (NoMouse) return;
	inregs.x.ax = 8;
	inregs.x.cx = min;
	inregs.x.dx = max;
	int86 (0x33,&inregs,&inregs);
}

/* Set the graphics cursor to that defined by the 'cursor' parameter */
void MoSetGrCursor (Curs *cursor)
{
union REGS inregs;
struct SREGS sregs;
unsigned far *p;

	if (NoMouse) return;
	p = cursor -> ScreenMask;
	inregs.x.ax = 9;
	inregs.x.bx = cursor -> XSpot;
	inregs.x.cx = cursor -> YSpot;
	inregs.x.dx = FP_OFF (p);
	sregs.ds = FP_SEG (p);
	sregs.es = FP_SEG (p);
	int86x (0x33,&inregs,&inregs,&sregs);
}

/* Defines the text cursor mode and shape.
 * Hw defines if you use hardware (1) or Software (0) cursor type.
 * If hardware cursor is selected the following parameters are
 * Start (SMask) and Stop (CMask) lines for the cursor.
 * If Software is selected, SMask is the Screen Mask parameter and
 * CMask is the Cursor Mask Parameter. */
void MoSetTextCur (int Hw,int SMask, int CMask)
{
union REGS inregs;
	if (NoMouse) return;
	inregs.x.ax = 10;
	inregs.x.bx = Hw;
	inregs.x.cx = SMask;
	inregs.x.dx = CMask;
	int86 (0x33,&inregs,&inregs);
}

/* Returns in Pos the motion counters (in 'mikey' units) from
 * the previous call to this function.
 * Buttons and Count are always 0 */
void MoMotionCnt (MPos *Pos)
{
union REGS inregs;
	if (NoMouse)
	{
		Pos -> X = 0;
		Pos -> Y = 0;
		Pos -> Buttons = 0;
		Pos -> Count = 0;
		return;
	}
	inregs.x.ax = 11;
	int86 (0x33,&inregs,&inregs);
	Pos -> X = inregs.x.cx;
	Pos -> Y = inregs.x.dx;
	Pos -> Buttons = 0;
	Pos -> Count = 0;
}

/* This call installs a routine that is called by the driver under
 * certain conditions.
 * Parameter mask is a bit mask. The bits are:
 *		0 -> Cursor Position Changes
 *		1 -> Left Button Pressed
 *		2 -> Left Button Released
 *		3 -> Right Button Pressed
 *		4 -> Right Button Released
 * And rout is the address of the 'action' routine.
 * This routine receives some data in its registers :
 * 		AX -> Conditions (as defined in the mask)
 *		BX -> Button State
 *		CX -> Cursor X
 *		DX -> Cursot Y
 */
void MoSetRespRout (void (*rout) (void),int mask)
{
union REGS inregs;
struct SREGS sregs;
void far (*r1) (void);

	if (NoMouse) return;
	r1 = (void far (*)(void)) rout;
	inregs.x.ax = 12;
	inregs.x.cx = mask;
	inregs.x.dx = FP_OFF (r1);
	sregs.es = FP_SEG (r1);
	sregs.ds = FP_SEG (r1);
	int86x (0x33,&inregs,&inregs,&sregs);
}

/* Turns Light Pen Emulation ON */
void MoLPEmu (void)
{
union REGS inregs;
	if (NoMouse) return;
	inregs.x.ax = 13;
	int86 (0x33,&inregs,&inregs);
}

/* Turns Light Pen Emulation Off */
void MoLPNoEmu (void)
{
union REGS inregs;
	if (NoMouse) return;
	inregs.x.ax = 14;
	int86 (0x33,&inregs,&inregs);
}

/* Sets the ratio between phisical (mikey) motion units and logical
 * (pixel) motion units both for x and y directions */
void MoSetRatio (int XRatio,int YRatio)
{
union REGS inregs;
	if (NoMouse) return;
	inregs.x.ax = 15;
	inregs.x.cx = XRatio;
	inregs.x.dx = YRatio;
	int86 (0x33,&inregs,&inregs);
}



